;   This file is a basic code template for assembly code generation   *
;   on the PICmicro PIC16F819. This file contains the basic code      *
;   building blocks to build upon.                                    *
;                                                                     *
;   If interrupts are not used all code presented between the ORG     *
;   0x004 directive and the label main can be removed. In addition    *
;   the variable assignments for 'w_temp' and 'status_temp' can       *
;   be removed.                                                       *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PICmicro data sheet for additional        *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    CHARGER.asm                                       *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:                                                          *
;    Company:                                                         *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                         *
;           NiMH                                         *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************

	list      p=16f819           ; list directive to define processor
	#include <p16F819.inc>        ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _WRT_ENABLE_OFF & _CPD_OFF & _CCP1_RB2 & _DEBUG_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_IO 

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.

;  Define Information
	#DEFINE	RS		PORTA, 7		; RA7 is RS line of LCD
	#DEFINE	E		PORTA, 6		; RA6 is E line of LCD
									; RB4-RB7 are D4-D7 of LCD
;  Macro
EStrobe 	MACRO					;  Strobe the "E" Bit
	BSF	E
	BCF	E
 ENDM

;********************************************************************************* 
; LCD Module commands
;*********************************************************************************
DISP_ON			EQU		0x00C			; Display on
CLR_DISP		EQU		0x001			; Clear the Display (    5 mS)
ENTRY_INC		EQU		0x006			;

LINE1           EQU     0x080           ;   
LINE2           EQU     0x0C0           ;   

LCD_DATA        EQU     PORTB			;   !!!


;***** VARIABLE DEFINITIONS
w_temp        	EQU     0x70        ; variable used for context saving 
status_temp   	EQU     0x71        ; variable used for context saving
pclath_temp	  	EQU		0x72	

	cblock 0x30
TMP	
Cnt
LoopCount
LoopCount2
CCP_temp
CCP2_temp
U_Ri							;     
TEMP
DCnt
DCnt2
Temp
Upow

BUFFER

huns
tens
ones

count1
count2
count3
I: 0, I_H, I_L					;   	
IT: 0, IT_H, IT_L
SI: 0, SI_H, SI_L				;   
V: 0, V_H, V_L					;  
VT: 0, VT_H, VT_L, VT_LL
SV: 0, SV_H, SV_L, SV_LL		;    
;MAXV							;      
	endc

;**********************************************************************
	ORG     0x000             ; processor reset vector
	goto    main              ; go to beginning of program
	

	ORG     0x004             ; interrupt vector location
	movwf   w_temp            ; save off current W register contents
	movf	STATUS,w          ; move status register into W register
	movwf	status_temp       ; save off contents of STATUS register
	MOVF    PCLATH, w
	MOVWF   pclath_temp
	CLRF    PCLATH


;	

    MOVF    pclath_temp, w
    MOVWF   PCLATH
	movf    status_temp,w     ; retrieve copy of STATUS register
	movwf	STATUS            ; restore pre-isr STATUS register contents
	swapf   w_temp,f
	swapf   w_temp,w          ; restore pre-isr W register contents
	retfie                    ; return from interrupt


main
    CLRF    STATUS          	; Do initialization (Bank 0)

	;     ,     .
    BSF     STATUS, RP0     	; Select Bank 1
	MOVLW	b'01110100'			;      8 
	MOVWF	OSCCON
    BCF     STATUS, RP0     	; Select Bank 0

	MOVLW	b'00000100'
	MOVWF	PORTA				;    A -  "0",  RA2 ( )
	CLRF	PORTB				;    B -  "0"

    BSF     STATUS, RP0     	; Select Bank 1
    MOVLW   b'00111011'
    MOVWF   TRISA           	;  A -  ,  RA2, RA6, RA7.
    MOVLW   b'00001011'
    MOVWF   TRISB           	;  B -    (RB2)  LCD_DATA (RB4-RB7).
    BCF     OPTION_REG,NOT_RBPU	;     B   
    BCF     STATUS, RP0     	; Select Bank 0

; -------    ----------
	CLRF	TMR2
	CLRF	T2CON				;    TMR2  1:1,  
	BSF		STATUS, RP0			;    1
	MOVLW	.59					;   (  33 )
	MOVWF	PR2					;       
	BCF		STATUS, RP0			;    0

	MOVLW	.25
	MOVWF	CCPR1L				;      50%
	MOVLW	0x0C			
	MOVWF	CCP1CON				;   CCP2/PWM2   . 
; -------------------------------------------

; --------   ----------------
    BSF     STATUS, RP0     	; Select Bank 1
    MOVLW   b'11000100'     	;  , T=32*Tosc, ... 
    MOVWF   ADCON1          	; ...   AN0, AN1  AN3,   - Vss  Vdd
    BCF     STATUS, RP0     	; Select Bank 0
    MOVLW   b'01000001'     	; T=16*Tosc,   - AN0,   , ...
    MOVWF   ADCON0          	; ...  .
; -------------------------------------------

	CALL	InitLCD				;  LCD

	BSF		T2CON, TMR2ON		;  

	CLRF	SI_H				;    
	CLRF	SI_L
	CLRF	SV_H				;    
	CLRF	SV_L
	CLRF	LoopCount
	CLRF	LoopCount2

	GOTO	chargeF				;  !!!

;**************   *****************
dicharge:
	CLRF	CCPR1L				;    ( )
	BCF		CCP1CON, 5
	BCF		CCP1CON, 4
	
  	BSF		PORTA, 2			;    (  )		
  	
  	MOVLW   LINE1+6        		;   7-    
	CALL    SendINS        
	MOVLW	'-'
	CALL	SendCHAR
	MOVLW	'-'
	CALL	SendCHAR
  	MOVLW   LINE2        		;   9-    
	CALL    SendINS        
	MOVLW	'-'
	CALL	SendCHAR
	MOVLW	'm'
	CALL	SendCHAR
	MOVLW	'A'
	CALL	SendCHAR

  	MOVLW   LINE2+4        		;   13-    
	CALL    SendINS        
	MOVLW	'D'
	CALL	SendCHAR
	MOVLW	'C'
	CALL	SendCHAR
	MOVLW	'h'
	CALL	SendCHAR
	MOVLW	'r'
	CALL	SendCHAR
	
l_dicharge:
	CALL	VMesure				;    
	
	MOVFW	V_H					;     
	MOVWF	VT_H
	MOVFW	V_L
	MOVWF	VT_L
	
; -------------     ------------
	RRF		VT_H, f
	RRF		VT_L, f
	CLRC
	RRF		VT_H, f
	RRF		VT_L, f
	CLRC
	MOVFW	VT_L
	MOVWF	Upow
; ------------------------------------------------------------------------

	MOVLW   LINE1        		;   1-     
	CALL    SendINS        

; ------   --------
	MOVFW	Upow
	MOVWF	BUFFER
	CALL	BINToBCD
	MOVFW	huns

	CALL    SendCHAR
	MOVFW	tens
	CALL    SendCHAR
	MOVLW	'.'
	CALL    SendCHAR
	MOVFW	ones
	CALL    SendCHAR
	MOVLW	'V'
	CALL    SendCHAR
;---------------------------------------

	MOVFW	Upow
	SUBLW	.82					;    8 
	BC		chargeF				;    

  	BCF		PORTA, 2			;   		
	CALL	Delay80ms
	CALL	Delay80ms
	CALL	Delay80ms
	CALL	Delay80ms
	CALL	Delay80ms
	CALL	Delay80ms
	CALL	Delay80ms
	CALL	Delay80ms
	CALL	Delay80ms
	CALL	Delay80ms
  	BSF		PORTA, 2			;   
	CALL	Delay80ms

	BTFSC	PORTB, 0			;     RB0 -    

	GOTO	l_dicharge
	
KeyRelease1:
	CALL	Delay40ms
	BTFSS	PORTB, 0
	GOTO	KeyRelease1		



;****************************************************

;***************   *****************
chargeF:
  	BSF		PORTA, 2			;    (  )		

	MOVLW	.2
	MOVWF	CCPR1L				;   

	CLRF	SV_H
	CLRF	SV_L
	CLRF	SV_LL
	
	CLRF	SI_H
	CLRF	SI_L

;	CLRF	MAXV

	CLRF	count1				;   
	CLRF	count2
	CLRF	count3

  	MOVLW   LINE2+4        		;   13-    
	CALL    SendINS        
	MOVLW	'C'
	CALL	SendCHAR
	MOVLW	'h'
	CALL	SendCHAR
	MOVLW	'r'
	CALL	SendCHAR
	MOVLW	'F'
	CALL	SendCHAR
	
; ********     *********
l_charge:
	CALL	IMesure
	MOVFW	I_H
	MOVWF	IT_H

;====================    ( ) ========================
	MOVFW	SI_H
	SUBWF	IT_H, f				;       
	CLRF	IT_L				;    
	CLRC						;    (  ).
;-----------------------------
;    8 (  256)  
	MOVLW   .8
	MOVWF   Cnt         		;     
loop_D1:
	BTFSC   IT_H, 7	    		;  I_H -    - 
	SETC                		;    -       
	RRF     IT_H, f	    		;    I   .
	RRF     IT_L, f  	  		;       (..   )
	DECFSZ  Cnt, f      		;        0
	GOTO    loop_D1
;-----------------------------
;-----------------------------
;    
	MOVFW   IT_H   	    		;    
	ADDWF   SI_H, f 	   		;  
	MOVFW   IT_L       			;   
	ADDWF   SI_L, f
	SKPNC               		;         -   
	INCF    SI_H, f    			;    "1"   
;-----------------------------
;=====================================================================================

;=============     0,4  ================
	CALL	composePWM
	
	MOVFW	I_H
	SUBLW	.82				;   0,4 V   1  (0,4 )
	BC		up_i
	TSTF	CCP_temp
	SKPZ
	DECF	CCP_temp, f
	GOTO	end_ipwm
up_i:	
	INCFSZ	CCP_temp, w		;    ...
	MOVWF	CCP_temp		; ...        FF
end_ipwm:

	CALL	decomposePWM
;=================================================================

	CALL	VMesure

	MOVFW	V_H
	MOVWF	VT_H
	MOVFW	V_L
	MOVWF	VT_L

; -------------     ------------
	RRF		VT_H, f
	RRF		VT_L, f
	CLRC
	RRF		VT_H, f
	RRF		VT_L, f
	CLRC
	MOVFW	VT_L
	MOVWF	Upow

;------      -------
	MOVFW	SI_H				; !!! ( I_H)
	MOVWF	IT_H

	RRF		IT_H, f
	CLRC
	RRF		IT_H, f
	CLRC
	RRF		IT_H, f
	CLRC
	RRF		IT_H, f
	CLRC
	MOVFW	IT_H
	SUBWF	Upow, f
;-------------------------------------------	

; -----   
	MOVFW	SV_H
	SUBWF	Upow, w				;       
	MOVWF	VT_H
	CLRF	VT_L
	CLRF	VT_LL
	CLRC

;-----------------------------
;    14 ()  
    MOVLW   .14					; ???
    MOVWF   Cnt         		;     
loop_D2:
    BTFSC   VT_H, 7	    		;  V_H -    - 
    SETC                		;    -       
    RRF     VT_H, f	    		;    I   .
    RRF     VT_L, f  	  		;       (..   )
    RRF		VT_LL, f
    DECFSZ  Cnt, f      		;        0
    GOTO    loop_D2
;-----------------------------

;-----------------------------
;    
    MOVFW   VT_H				;    
    ADDWF   SV_H, f				;  

    MOVFW   VT_L				;   
    ADDWF   SV_L, f
    SKPNC               		;         -   
    INCF    SV_H, f				;    "1"   

    MOVFW   VT_LL	     		;  ""  
    ADDWF   SV_LL, f
    BNC		end_sum_ll          	;         -   
    INCF    SV_L, f	    		;    "1"   
    SKPNZ						;    0 -   
    INCF	SV_H, f
end_sum_ll:    
;-----------------------------

; ------   --------
	MOVLW   LINE1        		;   1-     
	CALL    SendINS        

	MOVFW	SV_H				;   
	MOVWF	BUFFER
	CALL	BINToBCD
	MOVFW	huns

	CALL    SendCHAR
	MOVFW	tens
	CALL    SendCHAR
	MOVLW	'.'
	CALL    SendCHAR
	MOVFW	ones
	CALL    SendCHAR
	MOVLW	'V'
	CALL	SendCHAR
;---------------------------------------

;---------   ------------
	MOVLW   LINE1+6        		;   7-     
	CALL    SendINS        

	MOVFW	SI_H
	MOVWF	IT_H
	RRF		IT_H, f
	CLRC
	MOVFW	IT_H
	MOVWF	BUFFER

	CALL	BINToBCD
;     
	MOVFW	tens
	CALL    SendCHAR
	MOVFW	ones
	CALL    SendCHAR
	MOVLW   LINE2        		;   9-     
	CALL    SendINS        
	MOVLW	'0'
	CALL    SendCHAR
	MOVLW	'm'
	CALL	SendCHAR
	MOVLW	'A'
	CALL	SendCHAR

	BTFSS	PORTB, 0			;     RB0 -    
	GOTO	KeyRelease2
	
;---------------------------------------
;       
;	MOVFW	count3
;	SUBLW	.2
;	BC		end_gorb			;  30      
;	MOVFW	SV_H
;	SUBWF	MAXV, w
;	BC		chargeS				;      -  
;	MOVFW	SV_H
;	MOVWF	MAXV
;end_gorb:
;---------------------------------------

	INCF	count1, f
	BNZ		l_charge
	INCF	count2, f
	BNZ		l_charge
	INCF	count3, f
	MOVFW	count3
	SUBLW	.40					; ,    ~7 
	BC		l_charge			

KeyRelease2:					;    RB0
	CALL	Delay40ms
	BTFSS	PORTB, 0
	GOTO	KeyRelease2		


; ************    *************
chargeS:
  	BSF		PORTA, 2			;    (  )		

	MOVLW	.2
	MOVWF	CCPR1L				;   

	CLRF	SV_H
	CLRF	SV_L
	CLRF	SI_H
	CLRF	SI_L

  	MOVLW   LINE2+4        		;   13-    
	CALL    SendINS        
	MOVLW	'C'
	CALL	SendCHAR
	MOVLW	'h'
	CALL	SendCHAR
	MOVLW	'r'
	CALL	SendCHAR
	MOVLW	'S'
	CALL	SendCHAR

l_chargeS:
	CALL	IMesure
	MOVFW	I_H
	MOVWF	IT_H

;====================    ( ) ========================
	MOVFW	SI_H
	SUBWF	IT_H, f				;       
	CLRF	IT_L				;    
	CLRC						;    (  ).
;-----------------------------
;    8 (  256)  
	MOVLW   .8
	MOVWF   Cnt         		;     
loop_D4:
	BTFSC   IT_H, 7	    		;  I_H -    - 
	SETC                		;    -       
	RRF     IT_H, f	    		;    I   .
	RRF     IT_L, f  	  		;       (..   )
	DECFSZ  Cnt, f      		;        0
	GOTO    loop_D4
;-----------------------------
;-----------------------------
;    
	MOVFW   IT_H   	    		;    
	ADDWF   SI_H, f 	   		;  
	MOVFW   IT_L       			;   
	ADDWF   SI_L, f
	SKPNC               		;         -   
	INCF    SI_H, f    			;    "1"   
;-----------------------------
;=====================================================================================

;=============     0,03  ================
	CALL	composePWM

	MOVFW	I_H
	SUBLW	.6				;   0,03 V   1  (0,03 )
	BC		up_si
	TSTF	CCP_temp
	SKPZ
	DECF	CCP_temp, f
	GOTO	end_sipwm
up_si:	
	INCFSZ	CCP_temp, w		;    ...
	MOVWF	CCP_temp		; ...        FF
end_sipwm:

	CALL	decomposePWM
;=================================================================

	CALL	VMesure

	MOVFW	V_H
	MOVWF	VT_H
	MOVFW	V_L
	MOVWF	VT_L

; -------------     ------------
	RRF		VT_H, f
	RRF		VT_L, f
	CLRC
	RRF		VT_H, f
	RRF		VT_L, f
	CLRC
	MOVFW	VT_L
	MOVWF	Upow

;------      -------
	MOVFW	SI_H
	MOVWF	IT_H

	RRF		IT_H, f
	CLRC
	RRF		IT_H, f
	CLRC
	RRF		IT_H, f
	CLRC
	RRF		IT_H, f
	CLRC
	MOVFW	IT_H
	SUBWF	Upow, f
;-------------------------------------------	

; -----   
	MOVFW	SV_H
	SUBWF	Upow, w				;       
	MOVWF	VT_H
	CLRF	VT_L
	CLRC
;-----------------------------
;    8 (  256)  
    MOVLW   .8
    MOVWF   Cnt         		;     
loop_D3:
    BTFSC   VT_H, 7	    		;  V_H -    - 
    SETC                		;    -       
    RRF     VT_H, f	    		;    I   .
    RRF     VT_L, f  	  		;       (..   )
    DECFSZ  Cnt, f      		;        0
    GOTO    loop_D3
;-----------------------------
;-----------------------------
;    
    MOVFW   VT_H   	    		;    
    ADDWF   SV_H, f 	   		;  
    MOVFW   V_L       			;   
    ADDWF   SV_L, f
    SKPNC               		;         -   
    INCF    SV_H, f    			;    "1"   
;-----------------------------

; ------   --------
	MOVLW   LINE1        		;   1-     
	CALL    SendINS        

	MOVFW	SV_H				;   
	MOVWF	BUFFER
	CALL	BINToBCD
	MOVFW	huns

	CALL    SendCHAR
	MOVFW	tens
	CALL    SendCHAR
	MOVLW	'.'
	CALL    SendCHAR
	MOVFW	ones
	CALL    SendCHAR
	MOVLW	'V'
	CALL	SendCHAR
;---------------------------------------

;---------   ------------
	MOVLW   LINE1+6        		;   7-     
	CALL    SendINS        

	MOVFW	SI_H
	MOVWF	IT_H
	RRF		IT_H, f
	CLRC
	MOVFW	IT_H
	MOVWF	BUFFER

	CALL	BINToBCD
;     
	MOVFW	tens
	CALL    SendCHAR
	MOVFW	ones
	CALL    SendCHAR
	MOVLW   LINE2        		;   9-     
	CALL    SendINS        
	MOVLW	'0'
	CALL    SendCHAR
	MOVLW	'm'
	CALL	SendCHAR
	MOVLW	'A'
	CALL	SendCHAR

;---------------------------------------
	BTFSC	PORTB, 0			;     RB0 -    

	GOTO	l_chargeS

KeyRelease3:					;    RB0
	CALL	Delay40ms
	BTFSS	PORTB, 0
	GOTO	KeyRelease3		
	GOTO	dicharge	

;*************************************************************************
;* The LCD Module Subroutines                                            *
;*************************************************************************

;*************************************************************************;
;                 Initialize the LCD                                      ;
;*************************************************************************;
InitLCD:
	CALL	Delay40ms			; Wait 40 msecs before Reset 
	BCF		RS					; send an 8 bit instruction
	MOVLW	0x03				; Reset Command
	CALL	NybbleOut			; Send the Nybble
    CALL	Delay5ms			; Wait 5 msecs before Sending Again
	MOVLW	0x02				; Set 4 Bit Mode  !!!
	CALL	NybbleOut              
	;---   250 uS ---
	MOVLW	0xD0				 
	MOVWF	DCnt  
DLoopIL:
	DECFSZ	DCnt, f
	GOTO	DLoopIL
	;--------------------------
	MOVLW	0x028				; 4 bit, 2 Line, 5x7 font
	CALL	SendINS
	MOVLW	0x010				; display shift off
	CALL	SendINS
	MOVLW	CLR_DISP			; Clear the Display RAM
	CALL	SendINS
	CALL	Delay5ms			; Wait 5 msecs (Note, Can take up to 4.1 msecs)
	MOVLW	ENTRY_INC			;  increment cursor
	CALL	SendINS
	MOVLW	0x00C				;  display on cursor off
	CALL	SendINS
	MOVLW	LINE1				; 
	CALL	SendINS
	RETURN


;*************************************************************************;
;              Send an instruction in W out to the LCD                    ;
;*************************************************************************;
SendINS:                         
	MOVWF	Temp				; Save the Temporary Value
	SWAPF	Temp, w				; Send the High Nybble
	BCF		RS					;  
	CALL	NybbleOut
	MOVFW	Temp				; Send the Low Nybble
	CALL	NybbleOut
	RETURN

;*************************************************************************;
;              Send the character in W out to the LCD                     ;
;*************************************************************************;
SendASCII:						; Send nbr as ASCII character
	ADDLW	'0'
SendCHAR:						; Send the Character to the LCD
	MOVWF	Temp				; Save the Temporary Value
	SWAPF	Temp, w				; Send the High Nybble
	BSF		RS					;  
	CALL	NybbleOut
	MOVFW	Temp				; Send the Low Nybble
	BSF		RS					;  
	CALL	NybbleOut
	RETURN

;*************************************************************************;
;              Send the nibble in W out to the LCD                        ;
;*************************************************************************;
NybbleOut:                      ; Send a Nybble to the LCD
    ANDLW   0x0F            	;   
    MOVWF   TEMP	           	;       TEMP
	SWAPF	TEMP, f				;     
	MOVFW	LCD_DATA			;      LCD
    ANDLW   0x0F     			;  ,    LCD
    IORWF   TEMP, w	        	;   
    MOVWF   LCD_DATA           	;       LCD
	EStrobe						; Strobe out the LCD Data
	MOVLW	0xD0				;   250 uS
	MOVWF	DCnt  
DLoop1:
	DECFSZ	DCnt, f
	GOTO	DLoop1
	RETURN

;**************************************************************************
;       5, 40, 80 
;**************************************************************************
Delay80ms:
	MOVLW	0xFF				;     (256)
	GOTO	$+4 
Delay40ms:
	MOVLW	0x85				;     (133) 
	GOTO	$+2					;    
Delay5ms:
	MOVLW   0x10				;    ...
	MOVWF   DCnt2				; ...  (16)
DLoopOut:	
	MOVLW   0xFF            	;    ...
	MOVWF   DCnt          		; ...  (256)
DLoopIn:						;    ~0,3 mS
	DECFSZ  DCnt, f       		;   
	GOTO    DLoopIn	        	;  DCnt  0,  
	DECFSZ  DCnt2, f			;     
	GOTO    DLoopOut          	;  DCnt2  0,  
	RETURN


;*******************************************************************
; Routine: BINToBCD
;    3  
;   - BUFFER
;   - huns, tens, ones
;*******************************************************************
BINToBCD:
	MOVLW	.8
	MOVWF	Cnt
	CLRF	huns
	CLRF	tens
	CLRF	ones

BCDADD3:
	MOVLW	.5
	SUBWF	huns, 0
	BTFSC	STATUS, C
	CALL	ADD3HUNS

	MOVLW	.5
	SUBWF	tens, 0
	BTFSC	STATUS, C
	CALL	ADD3TENS	

	MOVLW	.5
	SUBWF	ones, 0
	BTFSC	STATUS, C
	CALL	ADD3ONES
	
	DECF	Cnt, 1
	CLRC
	RLF		BUFFER, 1
	RLF		ones, 1
	BTFSC	ones, 4
	CALL	CARRYONES
	RLF		tens, 1

	BTFSC	tens, 4
	CALL	CARRYTENS
	RLF		huns, 1
	CLRC

	MOVFW	Cnt
	SKPZ
	GOTO	BCDADD3

	MOVFW	huns
	ADDLW	0x30
	MOVWF	huns

	MOVFW	tens
	ADDLW	0x30
	MOVWF	tens
	
	MOVFW	ones
	ADDLW	0x30
	MOVWF	ones	

	RETURN

ADD3HUNS:
	MOVLW	.3
	ADDWF	huns, f
	RETURN	

ADD3TENS:
	MOVLW	.3
	ADDWF	tens, f
	RETURN

ADD3ONES:
	MOVLW	.3
	ADDWF	ones, f
	RETURN

CARRYONES:
	BCF		ones, 4
	SETC
	RETURN

CARRYTENS:
	BCF		tens, 4
	SETC
	RETURN

;***************************************************
;*                  *
;*     - V_H  V_L                *
;***************************************************
VMesure:
    MOVLW   b'01000001'     	; T=16*Tosc,   - AN0,   , ...
    MOVWF   ADCON0          	; ...  .
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	BSF		ADCON0, GO			;    
ADC_V_Loop:
	BTFSC	ADCON0, GO			;      
	GOTO	ADC_V_Loop
	MOVFW	ADRESH
	MOVWF	V_H
    BSF     STATUS, RP0     	; Select Bank 1	(   -   1 !!!)
	MOVFW	ADRESL				;     , ..     1 
    BCF     STATUS, RP0     	; Select Bank 0
	MOVWF	V_L
	RETURN
;***************************************************

;***************************************************
;*                  *
;*     - I_H                      *
;***************************************************
IMesure:
	MOVLW   b'01001001'     	; T=16*Tosc,   - AN1,   , ...
	MOVWF   ADCON0          	; ...  .
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	BSF		ADCON0, GO			;    
ADC_I_Loop:
	BTFSC	ADCON0, GO			;      
	GOTO	ADC_I_Loop
	BSF     STATUS, RP0     	; Select Bank 1	(   -   1 !!!)
	MOVFW	ADRESL				;     , ..     1 
	BCF     STATUS, RP0     	; Select Bank 0
	MOVWF	I_H
	RETURN
;***************************************************

;***************************************************
;*         *
;*    - CCP_temp                  *
;***************************************************
composePWM:
	MOVFW	CCPR1L			;      ...
	MOVWF	CCP_temp		; ...   
	CLRC
	RLF		CCP_temp, f		;       
	CLRC					;      
	RLF		CCP_temp, f
	CLRC					;      
	MOVFW	CCP1CON
	ANDLW	b'00110000'		;     5  4 
	MOVWF	CCP2_temp
	SWAPF	CCP2_temp, w	;   5  4   1  0
	IORWF	CCP_temp, f		;     
	RETURN
;***************************************************

;**********************************************************
;*        *
;*    - CCP_temp		                  *
;**********************************************************
decomposePWM:
	MOVFW	CCP_temp
	ANDLW	b'00000011'		;    
	MOVWF	CCP2_temp
	SWAPF	CCP2_temp, f	;        CCP1CON 
	MOVFW	CCP1CON
	ANDLW	b'11001111'		;      
	IORWF	CCP2_temp, f	;    CCP1CON    CCP2_temp
	CLRC
	RRF		CCP_temp, f
	CLRC
	RRF		CCP_temp, f
	CLRC					;    CCPR1L    CCP_temp
	MOVFW	CCP_temp
	MOVWF	CCPR1L
	MOVFW	CCP2_temp
	MOVWF	CCP1CON
	RETURN
;**********************************************************





	END                       ; directive 'end of program'

